<?php

include "Akrabat/Config/Abstract.php";

class Akrabat_Config_Ini extends Akrabat_Config_Abstract
{
    /**
     * Load a config file, overwriting any duplicate keys.
     * If $section is null, then the entire file is loaded.
     *
     * @param string $filename
     * @param string $section
     */
    public function load($filename, $section=null)
    {
        $iniArray = parse_ini_file($filename, true);
        if ($section) {
            if( isset($iniArray[$section])) {
                $this->_config = array_merge($this->_config, $this->processIncludes($iniArray, $section));
            } else {
                throw new Exception("No section '$section' in $filename'");
            }
        }
        else {
            foreach($iniArray as $section=>$value) {
                if (is_array($value)) {
                    if(!isset($this->_config[$section])) {
                        $this->_config[$section] = array();
                    }
                    $this->_config[$section] = array_merge($this->_config[$section], $this->processIncludes($iniArray, $section));
                } else {
                    $this->_config[$section] = $value;
                }
            }
        }
    }

    /**
     * Helper function to process the "include=" inheritance and then
     * process the "dot" single level sub-array syntax in each key.
     *
     * @param array $iniArray
     * @param string $section
     * @return array
     */
    protected function processIncludes($iniArray, $section)
    {
        $thisSection = $iniArray[$section];
        $config = array();
        foreach ($thisSection as $key => $value) {
            if ($key == 'include') {
                $sections = explode(',', $value);
                foreach ($sections as $s) {
                    if( isset($iniArray[$s])) {
                        $config = $this->array_merge_recursive($config, $this->processLevelsInSection($iniArray[$s]));
                    }
                }
                unset($thisSection[$key]);
            }
        }

        $r = $this->processLevelsInSection($thisSection);
        $config = $this->array_merge_recursive($config, $this->processLevelsInSection($thisSection), false);
        return $config;
    }


    /**
     * Helper function to handle single level namespace in the key.
     * Uses the first "." as the separator.
     *
     * @param array $section
     * @return array
     */
    protected function processLevelsInSection($section)
    {
        $config = array();
        foreach ($section as $key=>$value) {
            if (strpos($key, '.')) {
                $pieces = explode('.', $key, 2);
                if (!empty($pieces[1])) {
                    $config[$pieces[0]][$pieces[1]] = $value;
                } else {
                    $config[$key] = $value;
                }
            } else {
                $config[$key] = $value;
            }
        }
        return $config;
    }

    protected function array_merge_recursive($array1, $array2, $db=false)
    {
        if (is_array($array1) && is_array($array2)) {
            foreach ($array2 AS $key => $value) {
                if($db === true)
                {
                    zend::dump($key, 'amr: $key');
                    zend::dump($array1[$key], 'amr: $array1[$key]');
                }
                if(isset($array1[$key])) {
                    $array1[$key] = $this->array_merge_recursive($array1[$key], $value, $db);
                } else {
                    $array1[$key] = $value;
                }
            }
        } else {
            $array1 = $array2;
        }
        return $array1;
    }

}
?>